--[[
A UnitGroupHeader maintains a set of ActionButton and ActionButtonNone
instances.  It controls the state transitions for those buttons.  A
UnitGroupHeader also maintains a set of UnitGroupControl instances.  In WoW
2.01, there is a single UnitGroupControl for each UnitGroupHeader.  In WoW
2.02, there will be a UnitGroupControl for each unit in the associated
UnitGroup.

The UnitGroupHeader defines the valid states, and button-unit associations for
each of the ActionButton instances it maintains.  Each ActionButton is told
the state-unit associations, so that when the button is clicked, the action
associated with the button applies to the correct unit based on the current
state and those associations.

The UnitGroupHeader is associated with a single UnitGroup
The UnitGroupHeader maintains a list of ActionButton instances
The UnitGroupHeader maintains a single ActionButtonNone instance
The UnitGroupHeader maintains a single ActionSetLayout instance
--]]
uab.classes.UnitGroupHeader = uab.mixins.aceOO.Class()
uab.classes.UnitGroupHeader.nextState = 1

function uab.classes.UnitGroupHeader.NextState()
    local nextState = uab.classes.UnitGroupHeader.nextState
    uab.classes.UnitGroupHeader.nextState = uab.classes.UnitGroupHeader.nextState + 1

    return nextState
end

function uab.classes.UnitGroupHeader.prototype:init(group, numunits, actions)
	uab.classes.UnitGroupHeader.super.prototype.init(self)

	self.group = group
    self.numcontrols = numunits * uab.constants.controlsperunit
	self.controls = {}
    
	--[[
	Create the header frame.  We create one header from (the only one for the group), and
	multiple controls, and the loop counter is passed to the control to differentiate
    between control instances.
	--]]
    
    self.name = self.group.qualifiedname
    self.states = {}
    
    local state
    
    for idx = 1, self.numcontrols, 1 do
        state = uab.classes.UnitGroupHeader.NextState()
        if not self.states.min then
            self.states.min = state
        end
        self.controls[idx] = uab.classes.UnitGroupControl:new(self, state, group.name)
    end

    self.states.max = state

    uab.pdebug("Header: "..self.name.." : staterange = "..self.states.min.."-"..self.states.max)

	self.cancelbtn = uab.classes.ActionButtonNone:new(self)
	self.actionbtns = {}

    if actions then
    	for idx = 1, 24, 1 do
    	    if actions[idx] then
    	        self:SetAction(idx, actions[idx])
    		end
    	end
    end

	self:SecureHeaderChanged()
    self:PositionChanged()
    self:ScaleChanged()
end

function uab.classes.UnitGroupHeader.prototype:SecureHeaderChanged()
	local currentheader = self.secureheader
	
    if uab.db.profile.independent[self.group.name] then
        self.secureheader = uab.classes.SecureHeader:GetHeader(self.group.name)
    else
    	self.secureheader = uab.classes.SecureHeader:GetHeader("Global")
	end

    self.frame = self.secureheader.frame

	if self.secureheader ~= currentheader then
        self:IterateControls("SecureHeaderChanged")
        
		--[[
		Reparent all of this guys action buttons, and no action button, to
		the new header, if it is different.
		--]]
		self:SetButtonParent()
        
        --[[
        Reposition the buttons relative to the new header.
        --]]
        self:LayoutChanged()
		    
	    --[[
	    Make sure that all the state-anchor values are set properly on the
		header frame, since we changed headers
	    --]]
	    self:FastChangeUnitChanged()
	end
end

function uab.classes.UnitGroupHeader.prototype:SetButtonParent()

	if self.cancelbtn then
        self.cancelbtn:SetParent()
    end

	for _, actionbtn in pairs(self.actionbtns) do
		actionbtn:SetParent()
	end	
end

function uab.classes.UnitGroupHeader.prototype:IAmDead()
    --[[
    Set the header frame state attribute to 0 to (hopefully) cause our cancel
	and action buttons to hide (because of the state change to a hidden state).
	Since we are dead, we are no longer in combat, so this *should* be ok to
	do.
    --]]
    self.secureheader:IAmDead()
    self.cancelbtn:Hide()
    for _, btn in pairs(self.actionbtns) do
        btn:Hide()
    end
end

function uab.classes.UnitGroupHeader.prototype:SetAction(number, action)
    if not self.actionbtns[number] then
	    self.actionbtns[number] = uab.classes.ActionButton:new(self, action)
    else
	    self.actionbtns[number]:SetAction(action)
        self.actionbtns[number]:Enable()
    end
end

function uab.classes.UnitGroupHeader.prototype:LayoutChanged()
    if self.group.layout then
        self.group.layout:ArrangeButtons(self.cancelbtn, self.actionbtns)
    end
end

function uab.classes.UnitGroupHeader.prototype:IsRegisteredFrame(frame)
    return self:IterateControls("IsRegisteredFrame", frame)
end

function uab.classes.UnitGroupHeader.prototype:UnregisterFrame(frame)
    return self:IterateControls(
		function(control, frame)
			if control:IsRegisteredFrame(frame) then
				control:UnassignFrame()
				return true
			end
		end,
		frame)
end

function uab.classes.UnitGroupHeader.prototype:CanRegisterFrame(frame)
    local unit = SecureButton_GetModifiedUnit(frame)
    local canregister
    local groupname = self.group.name
    
	if not unit and groupname == "partyraid" then
		canregister = true
	elseif unit then
        canregister = (groupname == unit)
        if not canregister then
            if string.sub(unit, 1, 5) == "party" then
            	canregister = (groupname == "partyraid")
            elseif string.sub(unit, 1, 4) == "raid" then
            	canregister = (groupname == "partyraid")
            end
        end
	end

	return canregister and self:HaveAvailableControl()
end

function uab.classes.UnitGroupHeader.prototype:RegisterFrame(frame)
    local control = self:NextControl()

    if not control then return end

    control:AssignFrame(frame)
end

function uab.classes.UnitGroupHeader.prototype:ResetButtons()
	self.cancelbtn:Hide()

	for _, actionbtn in pairs(self.actionbtns) do
		actionbtn:Hide()
	end	
    
    self:LayoutChanged()
end

function uab.classes.UnitGroupHeader.prototype:Enable(force)
    if force or not self.forcedisable then
        self.forcedisable = nil
        self.cancelbtn:Enable()
	    self:IterateControls("Enable")
    end
end

function uab.classes.UnitGroupHeader.prototype:Disable(force)
    self.forcedisable = force or self.forcedisable
    self.cancelbtn:Disable()
    self:IterateControls("Disable")
end

function uab.classes.UnitGroupHeader.prototype:SetFrameGroupFrameEnabledStates()
    self:IterateControls("SetEnabledState")
end

function uab.classes.UnitGroupHeader.prototype:SwitchToSingleAction(action)
    self:IterateControls("SwitchToDirectAction", action)
end

function uab.classes.UnitGroupHeader.prototype:SwitchToButtons()
    self:IterateControls("SwitchToClickAction")
end

function uab.classes.UnitGroupHeader.prototype:TimerEvent(elapsed)
	for _, actionbtn in pairs(self.actionbtns) do
		actionbtn:TimerEvent(elapsed)
	end	
end

function uab.classes.UnitGroupHeader.prototype:ScaleChanged()
    self:IterateControls("ScaleChanged")

    if self.group:UsingDefaultScale() then
        uab.main.secureheader:SetStateScaleRange(self.states.min, self.states.max)
    else
        uab.main.secureheader:SetStateScaleRange(self.states.min, self.states.max, self.group:GetScale())
    end
end

function uab.classes.UnitGroupHeader.prototype:PositionChanged()
    self:IterateControls("PositionChanged")

    if self.group:UsingDefaultPosition() then
        uab.main.secureheader:SetStatePositionRange(self.states.min, self.states.max)
    else
        uab.main.secureheader:SetStatePositionRange(self.states.min, self.states.max, self.group:GetPosition().x, self.group:GetPosition().y, "cursor", "CENTER")
    end
end

function uab.classes.UnitGroupHeader.prototype:StayOpenChanged()
	for _, actionbtn in pairs(self.actionbtns) do
		actionbtn:StayOpenChanged()
	end	
end

function uab.classes.UnitGroupHeader.prototype:FastChangeUnitChanged()
    self:IterateControls("FastChangeUnitChanged")
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.UnitGroupHeader.prototype:HaveAvailableControl()
    return self:IterateControls(
		function(control)
			if not control.unitframe then
				return true
			end
		end)
end

function uab.classes.UnitGroupHeader.prototype:NextControl()
    return self:IterateControls(
		function(control)
			if not control.unitframe then
				return control
			end
		end)
end

function uab.classes.UnitGroupHeader.prototype:IterateControls(fcn, context1, context2, context3)
	for _, control in pairs(self.controls) do
		local result

		if type(fcn) == "string" then
		    result = control[fcn](control, context1, context2, context3)
		else
			result = fcn(control, context1, context2, context3)
		end
		
		if result then
		    return result
		end
	end
end
